import javax.swing.JOptionPane;
import javax.swing.Timer;
import java.util.Observer;
import java.util.Observable;
import javax.swing.JFrame;

/**
 * Create two Counters with associated Timers, and a "quit" dialog box so we can
 * end the program a little more neatly.
 * In this version, we take advantage of the Observer/Observable framework to
 * give one of the Counters TWO Observers: one that prints its value to System.out,
 * and one that displays its value in a graphics window.  Note that the
 * Counter class doesn't change at all; it doesn't need to know how it's
 * being displayed, or even how MANY ways it's being displayed.
 * We could add yet another Observer that emails the current value of the
 * Counter to Dr. Bloch, and Counter still wouldn't need to be changed.
 * @since version 3, Oct 10
 * @version 7, Oct 16
 */
public class Testbed
{
    private static JFrame frame;
    
    public static void test ()
    {
        Counter a = new Counter ();
        Counter b = new Counter ();
        final CounterComponent aWindow = new CounterComponent ();
        a.addObserver (new Observer () {
            public void update (Observable o, Object newValue) {
                System.out.println ("Counter a value: " + newValue);
            }
        });
        b.addObserver (new Observer () {
            public void update (Observable o, Object newValue) {
                System.out.println ("Counter b value: " + newValue);
            }
        });
        a.addObserver (aWindow);
        // note an Observable can have more than one Observer
        
        javax.swing.SwingUtilities.invokeLater (new Runnable () {
            public void run () {
                frame = new JFrame ("Counter a");
                frame.setSize (100, 100);
                frame.add (aWindow);
                frame.setVisible (true);
            }
        });

        Timer timerA = new Timer(1000, a.getIncrementer());
        Timer timerB = new Timer(2000, b.getIncrementer());
        timerB.start();
        timerA.start();
        JOptionPane.showMessageDialog(null, "Quit?");
        System.exit(0);
    }
}
